from pathlib import Path

import PIL.Image
import pytest

from material_color_utilities import (
    CustomColor,
    Theme,
    Variant,
    argb_from_hex,
    get_contrast_ratio,
    hex_from_argb,
    prominent_colors_from_image,
    theme_from_color,
    theme_from_image,
)


def test_argb_from_hex():
    assert argb_from_hex("#4285f4") == 0xFF4285F4
    assert argb_from_hex("#000000") == 0xFF000000
    assert argb_from_hex("#ffffff") == 0xFFFFFFFF


def test_hex_from_argb():
    assert hex_from_argb(0xFF4285F4) == "#4285f4"
    assert hex_from_argb(0xFF000000) == "#000000"
    assert hex_from_argb(0xFFFFFFFF) == "#ffffff"


def test_theme_from_source_color():
    custom_colors = [CustomColor("#4285F4", "Google Blue", True)]
    theme = theme_from_color("#FC03A3", 0.15, Variant.VIBRANT, custom_colors)
    assert isinstance(theme, Theme)
    assert theme.source == "#fc03a3"
    assert theme.schemes.light.primary == "#9f0065"
    assert theme.schemes.dark.background == "#1c1015"


def test_theme_from_image(assets_folder: Path):
    image = PIL.Image.open(assets_folder / "test.jpg")
    custom_colors = [CustomColor("#4285F4", "Google Blue", True)]
    theme = theme_from_image(image, 0.5, Variant.CONTENT, custom_colors)
    assert isinstance(theme, Theme)


def test_prominent_colors_from_image(assets_folder: Path):
    image = PIL.Image.open(assets_folder / "test.jpg")
    colors = prominent_colors_from_image(image, 5)
    assert isinstance(colors, list)
    assert len(colors) <= 5


def test_get_contrast_ratio():
    color1 = "#000000"
    color2 = "#FFFFFF"
    ratio = get_contrast_ratio(color1, color2)
    assert ratio == pytest.approx(21.0, rel=1e-2)


def test_dict():
    custom_colors = [CustomColor("#4285F4", "Google Blue", True)]
    theme = theme_from_color("#FC03A3", 8, Variant.VIBRANT, custom_colors)
    theme_dict = theme.dict()
    assert isinstance(theme_dict, dict)



import numpy as np
from PIL.Image import Image

from material_color_utilities._core import (
    CustomColor,
    Theme,
    Variant,
    prominent_colors_from_array,
    prominent_colors_from_array_argb,
    theme_from_array,
)


def image_to_argb(img: Image) -> np.ndarray:
    """Converts an image to ARGB flattened array with a length of 128 * 128 = 16384.

    Args:
        img: The input image to be converted.

    Returns:
        A 1D numpy array representing the image in ARGB format.
    """
    # Convert the image to 128x128 RGBA
    img = img.convert("RGBA")
    img.thumbnail((128, 128))

    # Extract channels: Alpha, Red, Green, Blue
    rgba_array = np.array(img)
    alpha = rgba_array[:, :, 3].astype(np.uint32) << 24
    red = rgba_array[:, :, 0].astype(np.uint32) << 16
    green = rgba_array[:, :, 1].astype(np.uint32) << 8
    blue = rgba_array[:, :, 2].astype(np.uint32)

    # Combine into ARGB format
    return (alpha | red | green | blue).ravel()


def prominent_colors_from_image_argb(img: Image, max_colors: int = 128) -> list[int]:
    """
    Extracts prominent ARGB colors from an image.

    The image is resized to 128x128 for performance before processing.
    More info:
        https://github.com/material-foundation/material-color-utilities/blob/main/concepts/color_extraction.md

    Args:
        img: The input image from which to extract colors.
        max_colors: A limit on the number of colors returned by the quantizer.
            Does not directly determine how many colors are returned, a reasonable default is 128.

    Returns:
        A list of prominent colors in ARGB format.
    """
    return prominent_colors_from_array_argb(image_to_argb(img), max_colors)


def prominent_colors_from_image(img: Image, max_colors: int = 128) -> list[str]:
    """
    Extracts prominent hex colors from an image.

    The image is resized to 128x128 for performance before processing.
    More info:
        https://github.com/material-foundation/material-color-utilities/blob/main/concepts/color_extraction.md

    Args:
        img: The input image from which to extract colors.
        max_colors: A limit on the number of colors returned by the quantizer.
            Does not directly determine how many colors are returned, a reasonable default is 128.

    Returns:
        A list of prominent colors in hex format.
    """
    return prominent_colors_from_array(image_to_argb(img), max_colors)


def theme_from_image(
    img: Image,
    contrast: float = 3,
    variant: Variant = Variant.VIBRANT,
    custom_colors: list[CustomColor] | None = None,
) -> Theme:
    """
    Generates a theme from an image, considering contrast, variant, and custom colors.

    The image is resized to 128x128 for performance before processing.

    Args:
        img: The input image used to generate the theme.
        contrast: The contrast level maintain between texts and their backgrounds.
        variant: Each variant is a set of design decisions on the assignment of color values from tonal palettes to color roles.
        custom_colors: A list of custom colors to be included in the theme.

    Returns:
        A theme generated from the image.
    """
    if custom_colors is None:
        custom_colors = []
    return theme_from_array(image_to_argb(img), contrast, variant, custom_colors)



from __future__ import annotations
from enum import StrEnum, auto
from typing import Any

import numpy as np


class Theme:
    argb: int
    source: int | None
    schemes: DynamicSchemeGroup
    contrast_level: float | None
    variant: Variant
    custom_colors: CustomColorGroup | None

    def __init__(self) -> None: ...

    def dict(self) -> dict[str, Any]: ...

    def __repr__(self) -> str: ...


class Hct:
    """The HCT (Hue-Chroma-Tone) color space is a cylindrical color space that is based on the Munsell color system.
    It is used to represent colors in a way that is more intuitive to humans.

    More info:
        https://github.com/material-foundation/material-color-utilities/blob/main/concepts/color_spaces.md
    """

    argb: int | None
    hex: str | None
    hue: float
    chroma: float
    tone: float

    def __init__(self) -> None: ...

    def __init__(self, argb: int) -> None: ...

    def __init__(self, hex: str) -> None: ...

    def dict(self) -> dict[str, Any]: ...

    def __repr__(self) -> str: ...


class DynamicSchemeGroup:
    light: DynamicScheme
    dark: DynamicScheme

    def __init__(self) -> None: ...

    def dict(self) -> dict[str, Any]: ...

    def __repr__(self) -> str: ...


class DynamicScheme:
    """The Dynamic Scheme comprises color attributes that are combined in a predetermined way to meet the needs
    of a user context or preference. It is essentially a mapping of color roles to color at specific tone in a
    tonal palette. For example, primary = 207H 80C 90T, onPrimary = 207H 80C 40T.

    More info:
    1. https://github.com/material-foundation/material-color-utilities/blob/main/concepts/dynamic_color_scheme.md
    2. https://github.com/material-foundation/material-color-utilities/blob/main/concepts/scheme_generation.md
    """

    source_color_hct: Hct
    variant: Variant
    is_dark: bool
    contrast_level: float

    primary_palette: TonalPalette
    secondary_palette: TonalPalette
    tertiary_palette: TonalPalette
    neutral_palette: TonalPalette
    neutral_variant_palette: TonalPalette
    error_palette: TonalPalette

    background: str
    surface: str
    surface_dim: str
    surface_bright: str
    surface_container_lowest: str
    surface_container_low: str
    surface_container: str
    surface_container_high: str
    surface_container_highest: str
    on_surface: str
    surface_variant: str
    on_surface_variant: str
    inverse_surface: str
    inverse_on_surface: str
    outline: str
    outline_variant: str
    shadow: str
    scrim: str
    surface_tint: str
    primary: str
    on_primary: str
    primary_container: str
    on_primary_container: str
    inverse_primary: str
    secondary: str
    on_secondary: str
    secondary_container: str
    on_secondary_container: str
    tertiary: str
    on_tertiary: str
    tertiary_container: str
    on_tertiary_container: str
    error: str
    on_error: str
    error_container: str
    on_error_container: str
    primary_fixed: str
    primary_fixed_dim: str
    on_primary_fixed: str
    on_primary_fixed_variant: str
    secondary_fixed: str
    secondary_fixed_dim: str
    on_secondary_fixed: str
    on_secondary_fixed_variant: str
    tertiary_fixed: str
    tertiary_fixed_dim: str
    on_tertiary_fixed: str
    on_tertiary_fixed_variant: str

    def __init__(self) -> None: ...

    def dict(self) -> dict[str, Any]: ...

    def __repr__(self) -> str: ...

    @staticmethod
    def get_rotated_hue(
        source_color: int, hues: list[float], rotations: list[float]
    ) -> float: ...


class TonalPalette:
    """A set of colors that share hue and chroma in HCT color space and vary in tones.

    From a perception perspective, we can say that they are "tones of the same color".
    MCU produces 6 tonal palettes: primary, secondary, tertiary, neutral, neutral variant, and error.
    Each comprises tones ranging from 0 to 100 that serves as the basis for mapping specific tones to specific roles.
    """

    hue: float
    chroma: float
    key_color: Hct | None

    def __init__(self) -> None: ...

    def __init__(self, argb: int) -> None: ...

    def __init__(self, hct: Hct) -> None: ...

    def __init__(self, hue: float, chroma: float) -> None: ...

    def __init__(self, hue: float, chroma: float, key_color: Hct) -> None: ...

    def get(self, tone: float) -> str: ...

    def get_argb(self, tone: float) -> int: ...

    def dict(self) -> dict[str, Any]: ...

    def __repr__(self) -> str: ...


class CustomColor:
    value_argb: int
    value: str
    name: str
    blend: bool

    def __init__(self) -> None: ...

    def __init__(self, value: int, name: str, blend: bool) -> None: ...

    def __init__(self, value: str, name: str, blend: bool) -> None: ...

    def dict(self) -> dict[str, Any]: ...

    def __repr__(self) -> str: ...


class CustomColorGroup:
    color: CustomColor
    value_argb: int
    value: str
    light: CustomColor
    dark: CustomColor

    def __init__(self) -> None: ...

    def dict(self) -> dict[str, Any]: ...

    def __repr__(self) -> str: ...


class ColorGroup:
    color_argb: int
    on_color_argb: int
    color_container_argb: int
    on_color_container_argb: int
    color: str
    on_color: str
    color_container: str
    on_color_container: str

    def __init__(self) -> None: ...

    def dict(self) -> dict[str, Any]: ...

    def __repr__(self) -> str: ...


class Variant(StrEnum):
    """Each variant is a set of design decisions on the assignment of color values from tonal palettes to color roles."""

    MONOCHROME = auto()
    NEUTRAL = auto()
    TONALSPOT = auto()
    VIBRANT = auto()
    EXPRESSIVE = auto()
    FIDELITY = auto()
    CONTENT = auto()
    RAINBOW = auto()
    FRUITSALAD = auto()


def argb_from_hex(hex: str) -> int:
    """Converts a hex color code string to its ARGB representation.

    In MCU, an sRGB color usually appears as a hexadecimal number in ARGB format: for example,
    #abcdef is 0xffabcdef. The leading ff means that this is an opaque color (alpha = 0xff).

    Args:
        hex: A hex color code string.

    Returns:
        The ARGB representation of the color.
    """
    ...


def hex_from_argb(argb: int) -> str:
    """Returns the hexadecimal representation of a color.

    In MCU, an sRGB color usually appears as a hexadecimal number in ARGB format: for example,
    #abcdef is 0xffabcdef. The leading ff means that this is an opaque color (alpha = 0xff).

    Args:
        argb: The ARGB color value.

    Returns:
        The hexadecimal representation of the color.
    """
    ...


def prominent_colors_from_array(image: np.ndarray, max_colors: int = 128) -> list[str]:
    """Returns the prominent hex colors from an image in the shape of a 2D array.

    The colors are sorted by score:
        https://github.com/material-foundation/material-color-utilities/blob/main/concepts/color_extraction.md#scoring

    More info:
        https://github.com/material-foundation/material-color-utilities/blob/main/concepts/color_extraction.md

    Args:
        image: A 1D array of ARGB values representing the image.
        max_colors: A limit on the number of colors returned by the quantizer.
            Does not directly determine how many colors are returned, a reasonable default is 128.

    Returns:
        A list of colors in hex format.
    """
    ...


def prominent_colors_from_array_argb(
    image: np.ndarray, max_colors: int = 128
) -> list[int]:
    """Returns the prominent ARGB colors from an image in the shape of a 1D array.

    The colors are sorted by score:
        https://github.com/material-foundation/material-color-utilities/blob/main/concepts/color_extraction.md#scoring

    More info:
        https://github.com/material-foundation/material-color-utilities/blob/main/concepts/color_extraction.md

    Args:
        image: A 1D array of ARGB values representing the image.
        max_colors: A limit on the number of colors returned by the quantizer.
            Does not directly determine how many colors are returned, a reasonable default is 128.

    Returns:
        A list of colors in ARGB format.
    """
    ...


def get_contrast_ratio(color1: str, color2: str) -> float:
    """Returns the contrast ratio of two colors.

    How to use contrast for accessibility:
        https://github.com/material-foundation/material-color-utilities/blob/main/concepts/contrast_for_accessibility.md

    Args:
        color1: The hex value of the first color.
        color2: The hex value of the second color.

    Returns:
        The contrast ratio between the two colors.
    """
    ...


def theme_from_color(
    source: str,
    contrast_level: float = 0.25,
    variant: Variant = Variant.VIBRANT,
    custom_colors: list[CustomColor] | None = None,
) -> Theme:
    """Returns a theme from a source color.

    More info:
        1: https://github.com/material-foundation/material-color-utilities/blob/main/concepts/color_extraction.md
        2. https://github.com/material-foundation/material-color-utilities/blob/main/concepts/dynamic_color_scheme.md
        3. https://github.com/material-foundation/material-color-utilities/blob/main/concepts/scheme_generation.md

    Args:
        source: The hex value of the source color.
        contrast_level: The contrast level (default is 0.25).
        variant: The variant type (default is 0).
        custom_colors: A list of custom colors (default is an empty list).

    Returns:
        A theme object
    """
    ...


def theme_from_argb_color(
    source: int,
    contrast_level: float = 0.25,
    variant: Variant = Variant.VIBRANT,
    custom_colors: list[CustomColor] | None = None,
) -> Theme:
    """Returns a theme from a source ARGB color.

    More info:
        1: https://github.com/material-foundation/material-color-utilities/blob/main/concepts/color_extraction.md
        2. https://github.com/material-foundation/material-color-utilities/blob/main/concepts/dynamic_color_scheme.md
        3. https://github.com/material-foundation/material-color-utilities/blob/main/concepts/scheme_generation.md

    Args:
        source: The ARGB value of the source color.
        contrast_level: The contrast level (default is 0.25).
        variant: The variant type (default is 0).
        custom_colors: A list of custom colors (default is an empty list).

    Returns:
        A theme object
    """
    ...


def theme_from_array(
    image: np.ndarray,
    contrast_level: float = 0.25,
    variant: Variant = Variant.VIBRANT,
    custom_colors: list[CustomColor] | None = None,
) -> Theme:
    """Returns a theme from an image.

    Args:
        image: A 1D array of ARGB values representing the image.
        contrast_level: The contrast level (default is 0.25).
        variant: The variant type (default is 0).
        custom_colors: A list of custom colors (default is an empty list).

    Returns:
        A theme object
    """
    ...



from material_color_utilities._core import (  # type: ignore
    CustomColor,
    CustomColorGroup,
    DynamicScheme,
    DynamicSchemeGroup,
    Hct,
    Theme,
    TonalPalette,
    Variant,
    argb_from_hex,
    get_contrast_ratio,
    hex_from_argb,
    prominent_colors_from_array,
    theme_from_argb_color,
    theme_from_color,
)
from material_color_utilities.image_utils import (
    image_to_argb,
    prominent_colors_from_image,
    theme_from_image,
)

__all__ = [
    "theme_from_image",
    "argb_from_hex",
    "hex_from_argb",
    "image_to_argb",
    "get_contrast_ratio",
    "theme_from_color",
    "theme_from_argb_color",
    "prominent_colors_from_image",
    "prominent_colors_from_array",
    "CustomColor",
    "Theme",
    "CustomColorGroup",
    "Variant",
    "TonalPalette",
    "DynamicScheme",
    "DynamicSchemeGroup",
    "Hct",
]
